@@ -1,6 +1,6 @@
 /*
  * This file is part of Gnome Subtitles.
- * Copyright (C) 2006-2010 Pedro Castro
+ * Copyright (C) 2006-2018 Pedro Castro
  *
  * Gnome Subtitles is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,15 +26,19 @@ using GnomeSubtitles.Ui.VideoPreview;
 using GnomeSubtitles.Ui.View;
 using Gtk;
 using Mono.Unix;
-using SubLib.Exceptions;
 using SubLib.Core.Domain;
+using SubLib.Exceptions;
+using SubLib.Util;
 using System;
 using System.IO;
 using System.Text;
+using System.Reflection;
 
 namespace GnomeSubtitles.Ui {
 
 public class MainUi {
+	private static Builder builder = null;
+
 	private Window window = null;
 	private Menus menus = null;
 	private Video video = null;
@@ -43,15 +47,15 @@ public class MainUi {
 	private Status status = null;
 
 	/* Constant strings */
-	private const string gladeFilename = "MainWindow.glade";
+	private const string uiFilename = "MainWindow.ui";
 	private const string iconFilename = "gnome-subtitles.svg";
-	
-	public MainUi (EventHandlers handlers, out Glade.XML glade) {
-		glade = new Glade.XML(null, gladeFilename, null, Base.ExecutionContext.TranslationDomain);
-		
-		window = glade.GetWidget("window") as Window;
+
+	public MainUi (EventHandlers handlers) {
+		builder = new Builder(uiFilename, Base.ExecutionContext.TranslationDomain);
+
+		window = builder.GetObject("window") as Window;
 		window.Icon = new Gdk.Pixbuf(null, iconFilename);
-		window.SetDefaultSize(Base.Config.PrefsWindowWidth, Base.Config.PrefsWindowHeight);
+		window.SetDefaultSize(Base.Config.ViewWindowWidth, Base.Config.ViewWindowHeight);
 
 		video = new Video();
 		view = new SubtitleView();
@@ -59,12 +63,12 @@ public class MainUi {
 		menus = new Menus();
 		status = new Status();
 
-		glade.Autoconnect(handlers);
+		builder.Autoconnect(handlers);
 		Base.InitFinished += OnBaseInitFinished;
-		
+
 		window.Visible = true;
     }
-    
+
 
 	/* Public properties */
 
@@ -75,54 +79,57 @@ public class MainUi {
 	public Menus Menus {
 		get { return menus; }
 	}
-	
+
 	public Video Video {
 		get { return video; }
 	}
-	
+
 	public SubtitleView View {
 		get { return view; }
 	}
-	
+
 	public SubtitleEdit Edit {
 		get { return edit; }
 	}
-	
+
 	public Status Status {
 		get { return status; }
 	}
 
-    
+
     /* Public Methods */
-    
+
+	public static Widget GetWidget (string name) {
+			return builder.GetObject(name) as Widget;
+	}
+
     /// <summary>Starts the GUI</summary>
     /// <remarks>A file is opened if it was specified as argument. If it wasn't, a blank start is performed.</summary>
     public void Start () {
-    	string[] args = Base.ExecutionContext.Args;
-    	if (args.Length > 0) {
-    		string subtitleFile = args[0];
-    		Uri videoUri = Base.Config.PrefsVideoAutoChooseFile ? VideoFiles.FindMatchingVideo(subtitleFile) : null;
+		string subtitleFilePath = GetSubtitleFileArg(Base.ExecutionContext.Args);
+		if (subtitleFilePath != null) {
+			Uri videoUri = Base.Config.VideoAutoChooseFile ? VideoFiles.FindMatchingVideo(subtitleFilePath) : null;
 			int codePage = GetFileOpenCodePageFromConfig();
-			Open(subtitleFile, codePage, videoUri);
+			Open(subtitleFilePath, codePage, videoUri);
 		}
     }
-    
+
     /// <summary>Closes the open file.</summary>
     public void Close () {
 		if (ToCloseAfterWarning())
 			Base.CloseDocument();
     }
-    
+
     /// <summary>Quits the application.</summary>
     public void Quit () {
 		if (ToCloseAfterWarning())
 			Base.Quit();
     }
-    
-	/// <summary>Kills the window in the most quick and unfriendly way.</summary>
-    public void Kill () {
-		window.Destroy();
-    }
+
+	///// <summary>Kills the window in the most quick and unfriendly way.</summary>
+  //  public void Kill () {
+		//window.Destroy();
+    //}
 
 	/// <summary>Creates a new subtitles document for the specified path.</summary>
 	/// <param name="path">The subtitles' filename. If it's an empty string, 'Unsaved Subtitles' will be used instead.</param>
@@ -133,7 +140,7 @@ public class MainUi {
 		else
 			Base.NewDocument();
     }
-    
+
     /// <summary>Shows the open dialog and possibly opens a subtitle.</summary>
     /// <remarks>If there's a document currently open with unsaved changes, a warning dialog
     /// is shown before opening the new file.</remarks>
@@ -158,9 +165,8 @@ public class MainUi {
 			Open(filename, -1, null);
 		}
     }
-        
+
     public void OpenVideo () {
-		
     	VideoOpenDialog dialog = Base.Dialogs.Get(typeof(VideoOpenDialog)) as VideoOpenDialog;
     	dialog.Show();
 		bool toOpen = dialog.WaitForResponse();
@@ -168,7 +174,7 @@ public class MainUi {
 			Base.OpenVideo(dialog.Uri);
 		}
     }
-    
+
     /// <summary>Executes a Save operation.</summary>
     /// <remarks>If the document hasn't been saved before, a SaveAs is executed.</remarks>
     /// <returns>Whether the file was saved.</returns>
@@ -185,7 +191,7 @@ public class MainUi {
     /// <remarks>After saving, the timing mode is set to the timing mode of the subtitle format using when saving.</remarks>
     /// <returns>Whether the file was saved.</returns>
     public bool SaveAs () {
-		FileSaveAsDialog dialog = Base.Dialogs.Get(typeof(FileSaveAsDialog)) as FileSaveAsDialog;
+		FileSaveDialog dialog = Base.Dialogs.Get(typeof(FileSaveDialog)) as FileSaveDialog;
 		FileProperties properties = ShowSaveAsDialog(dialog);
 		if (properties != null) {
 			Save(properties);
@@ -194,7 +200,7 @@ public class MainUi {
 		else
 			return false;
 	}
-	
+
 	/// <summary>Starts a new translation.</summary>
 	/// <remarks>If there's a translation open with unsaved changes, a warning dialog is shown.</remarks>
     public void TranslationNew () {
@@ -203,7 +209,7 @@ public class MainUi {
 
 		Base.NewTranslation();
     }
-    
+
     /// <summary>Shows the open translation dialog and possibly opens a file.</summary>
     /// <remarks>If there's a translation currently open with unsaved changes, a warning dialog
     /// is shown before opening the new file.</remarks>
@@ -217,7 +223,16 @@ public class MainUi {
     		OpenTranslation(filename, codePage);
     	}
     }
-    
+
+	/// <summary>Reloads the current translation file.</summary>
+	/// <remarks>If there's a translation currently open with unsaved changes, a warning dialog
+	/// is shown before opening the new file.</remarks>
+	public void TranslationReload () {
+		if (ToOpenTranslationAfterWarning()) {
+			OpenTranslation(Base.Document.TranslationFile.Path, Base.Document.TranslationFile.Encoding.CodePage);
+		}
+	}
+
     /// <summary>Executes a Save operation regarding the translation.</summary>
     /// <remarks>If the translation hasn't been saved before, a TranslationSaveAs is executed.</remarks>
     /// <returns>Whether the translation file was saved or not.</returns>
@@ -229,11 +244,11 @@ public class MainUi {
 		else
 			return TranslationSaveAs();
     }
-    
+	
     /// <summary>Executes a translation SaveAs operation.</summary>
     /// <returns>Whether the translation file was saved or not.</returns>
     public bool TranslationSaveAs () {
-		TranslationSaveAsDialog dialog = Base.Dialogs.Get(typeof(TranslationSaveAsDialog)) as TranslationSaveAsDialog;
+		FileTranslationSaveDialog dialog = Base.Dialogs.Get(typeof(FileTranslationSaveDialog)) as FileTranslationSaveDialog;
 		FileProperties properties = ShowSaveAsDialog(dialog);
 		if (properties != null) {
 			SaveTranslation(properties);
@@ -254,7 +269,7 @@ public class MainUi {
 
 
 	/* Private members */
-	
+
 	/// <summary>Opens a subtitle file, given its filename, code page and video filename.</summary>
 	/// <param name="path">The path of the subtitles file to open.</param>
 	/// <param name="codePage">The code page of the filename. To use autodetection, set it to -1.</param>
@@ -273,7 +288,7 @@ public class MainUi {
 				Open();
 		}
     }
-    
+
     /// <summary>Creates an <see cref="Encoding" /> from a code page.</summary>
     /// <param name="codePage">The code page.</param>
     /// <returns>The respective <see cref="Encoding" />, or null if codePage == -1.</returns>
@@ -323,7 +338,7 @@ public class MainUi {
 			}
 		}
 	}
-	
+
 	private void SaveTranslation (FileProperties fileProperties) {
 		try {
 			Base.Document.SaveTranslation(fileProperties);
@@ -333,22 +348,22 @@ public class MainUi {
 			errorDialog.Show();
 			bool toSaveAgain = errorDialog.WaitForResponse();
 	    	if (toSaveAgain)
-	    		TranslationSaveAs();			
+	    		TranslationSaveAs();
 		}
 	}
-	
+
 	/// <summary>Displays a SaveAs dialog and gets the chosen options as <cref="FileProperties" />.</summary>
 	/// <param name="dialog">The dialog to display.</param>
 	/// <returns>The chosen file properties, or null in case SaveAs was canceled.</returns>
-	private FileProperties ShowSaveAsDialog (SubtitleFileSaveAsDialog dialog) {
+	private FileProperties ShowSaveAsDialog (FileSaveDialog dialog) {
 		dialog.Show();
 		bool toSaveAs = dialog.WaitForResponse();
 		if (!toSaveAs)
 			return null;
-		
+
 		string path = dialog.Filename;
 		Encoding encoding = Encoding.GetEncoding(dialog.Encoding.CodePage);
-		SubtitleType subtitleType = dialog.SubtitleType;			
+		SubtitleType subtitleType = dialog.SubtitleType;
 		NewlineType newlineType = dialog.NewlineType;
 		TimingMode timingMode = Base.TimingMode;
 		return new FileProperties(path, encoding, subtitleType, timingMode, newlineType);
@@ -358,7 +373,7 @@ public class MainUi {
 	private bool ExistTextUnsavedChanges () {
 		return Base.IsDocumentLoaded && Base.Document.WasTextModified;
 	}
-	
+
 	/// <summary>Whether there are unsaved translation changes.</summary>
 	private bool ExistTranslationUnsavedChanges () {
 		return Base.IsDocumentLoaded && Base.Document.IsTranslationLoaded && Base.Document.WasTranslationModified;
@@ -375,7 +390,7 @@ public class MainUi {
    			SaveConfirmationDialog translationDialog = new SaveTranslationOnCloseConfirmationDialog();
    			toCreate = translationDialog.WaitForResponse();
    		}
-    	return toCreate; 
+    	return toCreate;
 	}
 
     /// <summary>Whether a new document should be created, after choosing the respective confirmation dialog.</summary>
@@ -391,7 +406,7 @@ public class MainUi {
    		}
    		return toCreate;
 	}
-	
+
 	/// <summary>Whether a document should be opened, after choosing the respective confirmation dialog.</summary>
 	private bool ToOpenAfterWarning () {
    		bool toCreate = true;
@@ -412,43 +427,44 @@ public class MainUi {
     		SaveConfirmationDialog dialog = new SaveTranslationOnNewTranslationConfirmationDialog();
    			return dialog.WaitForResponse();
    		}
-   		else
-    		return true; 
+
+		return true;
 	}
-	
+
 	/// <summary>Whether a translation should be opened, after choosing the respective confirmation dialog.</summary>
 	private bool ToOpenTranslationAfterWarning () {
    		if (ExistTranslationUnsavedChanges()) {
     		SaveConfirmationDialog dialog = new SaveTranslationOnOpenConfirmationDialog();
    			return dialog.WaitForResponse();
    		}
-   		else
-    		return true; 
+
+		return true;
 	}
-	
+
 	/// <summary>Whether a translation should be closed, after choosing the respective confirmation dialog.</summary>
     private bool ToCloseTranslationAfterWarning () {
    		if (ExistTranslationUnsavedChanges()) {
     		SaveConfirmationDialog dialog = new SaveTranslationOnCloseConfirmationDialog();
    			return dialog.WaitForResponse();
    		}
-   		else
-    		return true; 
+
+		return true;
 	}
-	
+
+	//FIXME use a header bar to show the title and subtitle
 	private void UpdateTitleModificationStatus (bool showFilename, bool modified) {
 		if (showFilename) {
-			string prefix = (modified ? "*" : String.Empty);
-			window.Title = prefix + Base.Document.TextFile.Filename + " - " + Base.ExecutionContext.ApplicationName;
+			window.Title = (modified ? "*" : "") + Base.Document.TextFile.Filename;
+		} else {
+			window.Title = Base.ExecutionContext.ApplicationName;
 		}
-		else window.Title = Base.ExecutionContext.ApplicationName;
 	}
 
 	private int GetFileOpenCodePageFromConfig () {
-		switch (Base.Config.PrefsDefaultsFileOpenEncoding) {
+		switch (Base.Config.FileOpenEncoding) {
 			case ConfigFileOpenEncoding.CurrentLocale: return Encodings.SystemDefault.CodePage;
 			case ConfigFileOpenEncoding.Fixed:
-				string encodingName = Base.Config.PrefsDefaultsFileOpenEncodingFixed;
+				string encodingName = Base.Config.FileOpenEncodingFixed;
 				EncodingDescription encodingDescription = EncodingDescription.Empty;
 				Encodings.Find(encodingName, ref encodingDescription);
 				return encodingDescription.CodePage;
@@ -456,26 +472,49 @@ public class MainUi {
 		}
 	}
 
+	private string GetSubtitleFileArg (string[] args) {
+		if (args.Length == 0) {
+			return null;
+		}
+
+		string file = args[0];
+		if (file == null) {
+			return null;
+		}
+
+		if (!Path.IsPathRooted(file)) {
+			try {
+				file = Path.GetFullPath(file);
+			}
+			catch (Exception e) {
+				Logger.Error(e, "Unable to read subtitle file path from argument 0");
+				return null;
+			}
+		}
+
+		return file;
+	}
+
 
 	/* Event members */
-	
+
 	private void OnBaseInitFinished () {
 		Base.DocumentLoaded += OnBaseDocumentLoaded;
 		Base.DocumentUnloaded += OnBaseDocumentUnloaded;
 	}
-	
+
 	private void OnBaseDocumentLoaded (Document document) {
 		UpdateTitleModificationStatus(true, false);
    		document.ModificationStatusChanged += OnBaseDocumentModificationStatusChanged;
     }
-    
+
     private void OnBaseDocumentUnloaded (Document document) {
     	if (document != null) {
     		document.ModificationStatusChanged -= OnBaseDocumentModificationStatusChanged;
     	}
     	UpdateTitleModificationStatus(false, false);
     }
-    
+
     private void OnBaseDocumentModificationStatusChanged (bool modified) {
     	UpdateTitleModificationStatus(true, modified);
 	}
